@@ -56,17 +56,17 @@ class PlaybackStatus:
 class Player(GObject.GObject):
     nextTrack = None
     timeout = None
-    playing = False
 
     __gsignals__ = {
         'playing-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
-        'playlist-item-changed': (GObject.SIGNAL_RUN_FIRST, None, (Gtk.ListStore, Gtk.TreeIter)),
+        'playlist-item-changed': (GObject.SIGNAL_RUN_FIRST, None, (Gtk.TreeModel, Gtk.TreeIter)),
         'current-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'playback-status-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'repeat-mode-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'volume-changed': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'prev-next-invalidated': (GObject.SIGNAL_RUN_FIRST, None, ()),
         'seeked': (GObject.SIGNAL_RUN_FIRST, None, (int,)),
+        'thumbnail-updated': (GObject.SIGNAL_RUN_FIRST, None, (str,)),
     }
 
     def __init__(self):
@@ -141,7 +141,6 @@ class Player(GObject.GObject):
         #and thus don't cause a message
         #In practice, self means only Gst.State.PLAYING and Gst.State.PAUSED are
         self._sync_playing()
-        self.emit('playing-changed')
 
     def _onBusError(self, bus, message):
         media = self.get_current_media()
@@ -192,16 +191,26 @@ class Player(GObject.GObject):
         self._sync_prev_next()
 
     def _get_next_track(self):
-        currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
+        if self.currentTrack and self.currentTrack.valid():
+            currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
+        else:
+            currentTrack = None
+
         nextTrack = None
+
         if self.repeat == RepeatType.SONG:
-            nextTrack = currentTrack
+            if currentTrack:
+                nextTrack = currentTrack
+            else:
+                nextTrack = self.playlist.get_iter_first()
         elif self.repeat == RepeatType.ALL:
-            nextTrack = self.playlist.iter_next(currentTrack)
-            if nextTrack is None:
+            if currentTrack:
+                nextTrack = self.playlist.iter_next(currentTrack)
+            if not nextTrack:
                 nextTrack = self.playlist.get_iter_first()
         elif self.repeat == RepeatType.NONE:
-            nextTrack = self.playlist.iter_next(currentTrack)
+            if currentTrack:
+                nextTrack = self.playlist.iter_next(currentTrack)
         elif self.repeat == RepeatType.SHUFFLE:
             nextTrack = self.playlist.get_iter_first()
             rows = self.playlist.iter_n_children(None)
@@ -224,17 +233,26 @@ class Player(GObject.GObject):
         return last
 
     def _get_previous_track(self):
-        currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
+        if self.currentTrack and self.currentTrack.valid():
+            currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
+        else:
+            currentTrack = None
+
         previousTrack = None
 
         if self.repeat == RepeatType.SONG:
-            previousTrack = currentTrack
+            if currentTrack:
+                previousTrack = currentTrack
+            else:
+                previousTrack = self.playlist.get_iter_first()
         elif self.repeat == RepeatType.ALL:
-            previousTrack = self.playlist.iter_previous(currentTrack)
-            if previousTrack is None:
+            if currentTrack:
+                previousTrack = self.playlist.iter_previous(currentTrack)
+            if not previousTrack:
                 previousTrack = self._get_iter_last()
         elif self.repeat == RepeatType.NONE:
-            previousTrack = self.playlist.iter_previous(currentTrack)
+            if currentTrack:
+                previousTrack = self.playlist.iter_previous(currentTrack)
         elif self.repeat == RepeatType.SHUFFLE:
             previousTrack = self.playlist.get_iter_first()
             rows = self.playlist.iter_n_children(None)
@@ -247,22 +265,30 @@ class Player(GObject.GObject):
             return None
 
     def has_next(self):
-        if not self.currentTrack:
+        if not self.playlist or self.playlist.iter_n_children(None) < 1:
+            return False
+        elif not self.currentTrack:
             return False
         elif self.repeat in [RepeatType.ALL, RepeatType.SONG, RepeatType.SHUFFLE]:
             return True
-        else:
+        elif self.currentTrack.valid():
             tmp = self.playlist.get_iter(self.currentTrack.get_path())
             return self.playlist.iter_next(tmp) is not None
+        else:
+            return True
 
     def has_previous(self):
-        if not self.currentTrack:
+        if not self.playlist or self.playlist.iter_n_children(None) < 1:
+            return False
+        elif not self.currentTrack:
             return False
         elif self.repeat in [RepeatType.ALL, RepeatType.SONG, RepeatType.SHUFFLE]:
             return True
-        else:
+        elif self.currentTrack.valid():
             tmp = self.playlist.get_iter(self.currentTrack.get_path())
             return self.playlist.iter_previous(tmp) is not None
+        else:
+            return True
 
     def _get_playing(self):
         ok, state, pending = self.player.get_state(0)
@@ -274,6 +300,10 @@ class Player(GObject.GObject):
         else:
             return False
 
+    @property
+    def playing(self):
+        return self._get_playing()
+
     def _sync_playing(self):
         image = self._pauseImage if self._get_playing() else self._playImage
         if self.playBtn.get_image() != image:
@@ -329,6 +359,7 @@ class Player(GObject.GObject):
     def _on_cache_lookup(self, pixbuf, path, data=None):
         if pixbuf is not None:
             self.coverImg.set_from_pixbuf(pixbuf)
+        self.emit('thumbnail-updated', path)
 
     def play(self):
         if self.playlist is None:
@@ -337,7 +368,11 @@ class Player(GObject.GObject):
         if self.player.get_state(1)[1] != Gst.State.PAUSED:
             self.stop()
 
-        self.load(self.get_current_media())
+        media = self.get_current_media()
+        if not media:
+            return
+
+        self.load(media)
 
         self.player.set_state(Gst.State.PLAYING)
         self._update_position_callback()
@@ -345,6 +380,7 @@ class Player(GObject.GObject):
             self.timeout = GLib.timeout_add(1000, self._update_position_callback)
 
         self.emit('playback-status-changed')
+        self.emit('playing-changed')
 
     def pause(self):
         if self.timeout:
@@ -513,13 +549,25 @@ class Player(GObject.GObject):
     def _sync_repeat_image(self):
         icon = None
         if self.repeat == RepeatType.NONE:
-            icon = 'media-playlist-consecutive-symbolic'
+            if Gtk.Widget.get_default_direction() is not Gtk.TextDirection.RTL:
+                icon = 'media-playlist-consecutive-symbolic'
+            else:
+                icon = 'media-playlist-consecutive-rtl-symbolic'
         elif self.repeat == RepeatType.SHUFFLE:
-            icon = 'media-playlist-shuffle-symbolic'
+            if Gtk.Widget.get_default_direction() is not Gtk.TextDirection.RTL:
+                icon = 'media-playlist-shuffle-symbolic'
+            else:
+                icon = 'media-playlist-shuffle-rtl-symbolic'
         elif self.repeat == RepeatType.ALL:
-            icon = 'media-playlist-repeat-symbolic'
+            if Gtk.Widget.get_default_direction() is not Gtk.TextDirection.RTL:
+                icon = 'media-playlist-repeat-symbolic'
+            else:
+                icon = 'media-playlist-repeat-rtl-symbolic'
         elif self.repeat == RepeatType.SONG:
-            icon = 'media-playlist-repeat-song-symbolic'
+            if Gtk.Widget.get_default_direction() is not Gtk.TextDirection.RTL:
+                icon = 'media-playlist-repeat-song-symbolic'
+            else:
+                icon = 'media-playlist-repeat-song-rtl-symbolic'
 
         self.repeatBtnImage.set_from_icon_name(icon, Gtk.IconSize.MENU)
         self.emit('repeat-mode-changed')
@@ -595,7 +643,7 @@ class Player(GObject.GObject):
         self.emit('volume-changed')
 
     def get_current_media(self):
-        if not self.currentTrack:
+        if not self.currentTrack or not self.currentTrack.valid():
             return None
         currentTrack = self.playlist.get_iter(self.currentTrack.get_path())
         return self.playlist.get_value(currentTrack, self.playlistField)
